//-< Persistent.java >-----------------------------------------------*--------*
// GOODS                      Version 2.02       (c) 1998  GARRET    *     ?  *
// (Generic Object Oriented Database System)                         *   /\|  *
// Java Program Interface                                            *  /  \  *
//                          Created:      1-Oct-98    K.A. Knizhnik  * / [] \ *
//                          Last update: 16-Nov-98    K.A. Knizhnik  * GARRET *
//-------------------------------------------------------------------*--------*
// Base class for all persistent objects
//-------------------------------------------------------------------*--------*

package goodsjpi;

public class Persistent { 
    public Metaobject metaobject;

    ClassDescriptor desc; 

    boolean         invalidated; 
    byte[]          data; // packed object data (as in the storage) 

    int             opid;
    Storage         storage;
    
    Persistent      next; 
    Persistent      prev;

    int             accessCount;
    int             state;

    final static int DIRTY      = 0x0001; // object was changed
    final static int RAW        = 0x0002; // object has to be loaded from server
    final static int CACHED     = 0x0004; // object is placed in LRU cache
    final static int DESTRUCTED = 0x0008; // object was thrown away
    final static int TRANSREAD  = 0x0010; // object was accessed for reading in transaction
    final static int TRANSWRITE = 0x0020; // object was accessed for writing in transaction
    final static int USEFUL     = 0x0040; // object was access at least twice not sequentially
    final static int ACCESSED   = 0x0080; // object was accessed at least once
    final static int SLOCKED    = 0x0100; // object is shared locked
    final static int XLOCKED    = 0x0200; // object is exslusivly locked
    final static int NEW        = 0x0400; // object was just created
    final static int OPTIMISTIC = 0x0800; // object under optimistic metaobject control and has to be validated at sserver
    final static int NOTIFY     = 0x1000; // notification about object change should be send

    public Object clone() throws CloneNotSupportedException { 
	Persistent p = (Persistent)super.clone();
	p.invalidated = false;
	p.data = null;
	p.opid = 0;
	p.storage = null;	
	p.next = null; 
	p.prev = null;	
	p.accessCount = 0;
	p.state = 0;
	return p;
    }

    final protected void finalize() { 
	if (opid != 0) { 
	    metaobject.forgetObject(this);
	}
    }

    public synchronized void attachToStorage(Database db, int sid) { 
	Assert.that("persistent object can't change its location", opid == 0);
	storage = db.storages[sid];
    }

    public synchronized void clusterWith(Persistent obj) { 
	Assert.that("persistent object can't change its location", opid == 0);
	storage = obj.storage;
    }

    public Database getDatabase() { 
	return storage != null ? storage.database : null; 
    }

    /**
     * Get the dabase assigned id of the object
     */
    public int getOid() {
	return opid;
    }

    public int getStorageId() { 
	return storage != null ? storage.id : -1; 
    }

    public int hashCode() { 
	Assert.that(opid != 0);
	return opid;
    }

    protected Persistent() {
	desc = ClassDescriptor.lookup(getClass());
	metaobject = desc.defaultMetaobject;
	metaobject.preDaemon(this, Metaobject.MUTATOR|Metaobject.CONSTRUCTOR);
    }

    protected Persistent(Metaobject metaobject) {
	this.metaobject = metaobject;
    }

    protected void onLoad() {
	
    }

    public static Metaobject defaultMetaobject = 
        new PessimisticRepeatableReadMetaobject();
}    

