//-< ObjectClosure.java >-----------------------------------------------------*
// GOODS                      Version 2.02       (c) 1998  GARRET    *     ?  *
// (Generic Object Oriented Database System)                         *   /\|  *
// GOODS Persistent Class Library                                    *  /  \  *
//                          Created:     20-Oct-98    K.A. Knizhnik  * / [] \ *
//                          Last update: 20-Oct-98    K.A. Knizhnik  * GARRET *
//----------------------------------------------------------------------------*
// Persistent class for storing closure of serialized transient objects
//----------------------------------------------------------------------------*

package goodslib;
import  goodsjpi.*;

import java.io.*;

/** 
 * This class can be used to save in persistent storage cluster of 
 * transient Java objects using serialization mecahnism. Class
 * java.io.ObjectOutputStream is used to pack all objects reachable from
 * some specified root object into stream of bytes. Data is stored in 
 * GOODS persistent object as array of bytes. It can be restored lately
 * by java.io.ObjectInputStream (classes of all packed objects should be
 * available in application). 
 */
public class ObjectClosure extends Persistent implements Comparable { 
    protected byte[] buf;
    protected int objversion;

    transient boolean objref_loaded;
    transient Object objref;
    transient int refversion;
   
    public ObjectClosure() {
	store(null);
    }

    /** 
     * Pack objects into array of bytes to be placed in storage.
     * All objects accessible by refernces from root objects will be placed
     * in this object closure. 
     */
    public synchronized void store(Object root) { 
	try { 
	    Metaobject.modify();
	    if(root != null) {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		ObjectOutputStream clu = new ObjectOutputStream(out);
		clu.writeObject(root);
		clu.close();
		buf = out.toByteArray();
	    } else {
		buf = new byte[1];
		buf[0]=0;
	    }
	    objversion++;
	    objref_loaded=true;
	    objref=root;
	    refversion=objversion;
	} catch (Exception x) { 
	    x.printStackTrace(System.err);
	    throw new Error(x.getMessage());
	}
    }

    /**
     * Extract objects from object closure and returns reference to root object
     * and return reference to root object. After been extracted from the 
     * closure, these objects will have no differenecs with normal Java
     * transparent objects. 
     */
    public synchronized Object extract() { 
	if(!objref_loaded || refversion != objversion) {
	    try { 
		if(buf != null && buf.length > 1) {
		    ByteArrayInputStream input = new ByteArrayInputStream(buf);
		    ObjectInputStream in = new ObjectInputStream(input);
		    objref=in.readObject();
		} else {
		    objref=null;
		}
		objref_loaded=true;
		refversion=objversion;
	    } catch (Exception x) { 
		x.printStackTrace(System.err); 
		throw new Error(x.getMessage());
	    }
	}
	return objref;
    }

    /**
     * Create new instance of object closure given specified root object.
     */
    public ObjectClosure(Object root) { 
	store(root);
    }

    public int compareTo(Object o) {
        Object myobj=extract();

	if(o instanceof ObjectClosure) {
	    o=((ObjectClosure)o).extract();
	}
	
	if(myobj==null) {
	    return -1;
	}

        if(o==null) {
            return 1;
        }

	if(myobj instanceof Comparable) {
	    return ((Comparable)myobj).compareTo(o);
        } else if(o instanceof Comparable && !(o instanceof ObjectClosure)) {
	    return -((Comparable)o).compareTo(myobj);
	} else {
	    return 0;
	}
    }

    public boolean equals(Object o) {
        Object myobj=extract();

	if(o instanceof ObjectClosure) {
	    o=((ObjectClosure)o).extract();
	}
	
	if(myobj==null && o==null) {
	    return true;
	}

        if(myobj==null || o==null) {
            return false;
        }

	return myobj.equals(o);
    }

    public int hashCode() {
	Object myval=extract();
	if(myval != null) {
	    return myval.hashCode();
	} else {
	    return super.hashCode();
	}
    }
}
