/**
 * CollectionTest.java
 *
 *
 * Created: Tue Dec  4 12:06:41 2001
 *
 * @author <a href="mailto:hotari@iki.fi">Lari Hotari</a>
 * @version
 */

import junit.framework.*;
import goodslib.*;
import goodsjpi.*;
import java.util.*;
import java.lang.reflect.*;

public class CollectionTest extends TestCase {
    Database db;
    PersistentHashMap root;

    public CollectionTest (String test){
	super(test);
    }

    public void setUp() {
	db = new Database();
        if (!db.open("omatest.cfg")) { 
	    fail("Could not open database");
	}
	root = (PersistentHashMap)db.getRoot();
	if(root==null) {
	    root = new PersistentHashMap();
	    db.setRoot(root);
	}
    }

    public void tearDown() {
	if(db != null) {
	    db.close();
	}
    }

    public void testHash() {
	System.out.println("1 = " + (new PersistentLong(1)).hashCode());
	System.out.println("2 = " + (new PersistentLong(2)).hashCode());
	System.out.println("1 = " + (new PersistentLong(1)).hashCode());
	System.out.println("2 = " + (new PersistentLong(2)).hashCode());
    }

    public void testSaveBig() throws Exception {
	StringBuffer sb=new StringBuffer();
	for(int i=0;i < 32768;i++) {
	    sb.append('a');
	}
	String bigstr=sb.toString();

	List list=new PersistentLinkedList();
	root.put("biglist", list);
	for(int i=0;i < 1000;i++) {
	    System.out.print(".");
	    list.add(bigstr + "jee" + i);
	}
    }

    public void testSaveBigOld() throws Exception {
	StringBuffer sb=new StringBuffer();
	for(int i=0;i < 32768;i++) {
	    sb.append('a');
	}
	String bigstr=sb.toString();

	List list=new PersistentArrayList(1000);
	root.put("biglist", list);
	for(int i=0;i < 1000;i++) {
	    System.out.print(".");
	    list.add(bigstr + "jee" + i);
	}
    }

    public void testSaveSorted() throws Exception {
	Set sortedset=new PersistentTreeSet();
	root.put("sorted", sortedset);
	Random random=new Random();
	for(int i=0;i < 1000;i++) {
	    System.out.print(".");
	    sortedset.add(String.valueOf((char)('a' + random.nextInt(26))) + "jee" + i);
	}
    }

    public void testSaveSortedNumbers() throws Exception {
	Set sortedset=new PersistentTreeSet();
	root.put("sorted", sortedset);
	Random random=new Random();
	for(int i=0;i < 1000;i++) {
	    System.out.print(".");
	    sortedset.add(new Integer(random.nextInt()));
	}
    }

    public void testLoadSorted() throws Exception {
	Set sortedset=(Set)root.get("sorted");
	for(Iterator i=sortedset.iterator();i.hasNext();) {
	    System.out.println(i.next());
	}
    }

    public void testLoadBig() throws Exception {
	List list=(List)root.get("biglist");
	System.out.println("size:"+list.size());
    }

    public void testSaveTree() throws Exception {
	try {
	    Map map=new PersistentHashMap();
	    root.put("treemap",map);
	    List list=new PersistentArrayList();
	    list.add("yy");
	    list.add("kaa");
	    list.add("koo");
	    map.put("lista", list);
	    List list2=new PersistentArrayList();
	    list2.add("jepe");
	    list2.add("jepejee");
	    map.put("lista2",list2);
	    map.put("testi","jee");
	} catch (Exception e) {
	}
    }


    public void testSaveMap() throws Exception {
	Map map=new PersistentHashMap();
	root.put("mappi",map);
	map.put(new Long(1), "ykknen");
	map.put(new Long(2), "kakkonen");
	map.put("1", "ykknen");
	map.put("2", "kakkonen");
    }

    public void testLoadMap() throws Exception {
	Map map=(Map)root.get("mappi");
	System.out.println("1 = " + map.get(new Long(1)));
	System.out.println("2 = " + map.get(new Long(2)));
	System.out.println("'1' = " + map.get("1"));
	System.out.println("'2' = " + map.get("2"));
	printCollection(map,0);
    }


    public void testUpdateTree() throws Exception {
	Map map=(Map)root.get("treemap");
	List list=(List)map.get("lista2");
	Map submap=new PersistentHashMap();
	submap.put("jeps","jeejeejee");
	list.add(submap);
    }

    public void testLoadTree() throws Exception {
	Map map=(Map)root.get("treemap");
	printCollection(map,0);
    }

    public void testSaveNumber() throws Exception {
	NumberObj numobj=new NumberObj();
	root.put("numobj",numobj);
	System.out.println(numobj);
    }

    public void testLoadNumber() throws Exception {
	NumberObj numobj=(NumberObj)root.get("numobj");
	System.out.println(numobj);
    }

    public void printCollection(Object obj, int indent) {
	if(obj instanceof Map) {
	    printMap((Map)obj, indent);
	} else if (obj instanceof List) {
	    printList((List)obj, indent);
	}
    }
    
    public void printMap(Map map, int indent) {
	String indentstr="";
	for(int i=0;i < indent;i++) {
	    indentstr+="\t";
	}
	System.out.println(indentstr + "map:" + map);
	for(Iterator it=map.entrySet().iterator();it.hasNext();) {
	    Map.Entry entry=(Map.Entry)it.next();
	    Object key=entry.getKey();
	    Object value=entry.getValue();
	    System.out.println(indentstr + "key:" + key + " value:" + value);
	    printCollection(value,indent+1);
	}
    }

    public void printList(List list, int indent) {
	String indentstr="";
	for(int i=0;i < indent;i++) {
	    indentstr+="\t";
	}
	System.out.println(indentstr + "list:" + list);
	System.out.println(indentstr + "size:" + list.size());
	for(Iterator it=list.iterator();it.hasNext();) {
	    Object value=it.next();
	    System.out.println(indentstr + "value:" + value);
	    printCollection(value,indent+1);
	}
    }

    public void testReflect() throws Exception {
	FieldTest fldtest=new FieldTest();
	Field valfld=FieldTest.class.getField("val");
	System.out.println("val:" + valfld.get(fldtest));
	System.out.println("valtype:" + valfld.getType());
	valfld.set(fldtest, new Long(234));
	System.out.println("val:" + valfld.get(fldtest));
    }

    public class FieldTest {
	public Long val;

	public FieldTest() {
	    val=new Long(123);
	    //val=123;
	}
    }
}// CollectionTest
