package goodsutil;
/**
 * GoodsConfig.java
 *
 *
 * Created: Tue Dec  4 12:06:41 2001
 *
 * @author <a href="mailto:hotari@iki.fi">Lari Hotari</a>
 * @version
 */

import java.io.*;
import java.util.*;
import org.apache.regexp.*;

public class GoodsConfig extends Properties {
    File cfgfile;

    public static final String MEMMGR_INIT_MAP_FILE_SIZE="memmgr.init_map_file_size";
    public static final String MEMMGR_INIT_INDEX_FILE_SIZE="memmgr.init_index_file_size";
    public static final String MEMMGR_GC_INIT_TIMEOUT="memmgr.gc_init_timeout";
    public static final String MEMMGR_GC_RESPONSE_TIMEOUT="memmgr.gc_response_timeout";
    public static final String MEMMGR_GC_INIT_ALLOCATED="memmgr.gc_init_allocated";
    public static final String MEMMGR_GC_INIT_IDLE_PERIOD="memmgr.gc_init_idle_period";
    public static final String MEMMGR_GC_INIT_MIN_ALLOCATED="memmgr.gc_init_min_allocated";
    public static final String MEMMGR_GC_GREY_SET_THRESHOLD="memmgr.gc_grey_set_threshold";
    public static final String MEMMGR_MAX_DATA_FILE_SIZE="memmgr.max_data_file_size";
    public static final String MEMMGR_MAX_OBJECTS="memmgr.max_objects";
    public static final String MEMMGR_INDEX_FILE_NAME="memmgr.index_file_name";
    public static final String MEMMGR_MAP_FILE_NAME="memmgr.map_file_name";
    public static final String TRANSMGR_SYNC_LOG_WRITES="transmgr.sync_log_writes";
    public static final String TRANSMGR_PERMANENT_BACKUP="transmgr.permanent_backup";
    public static final String TRANSMGR_MAX_LOG_SIZE="transmgr.max_log_size";
    public static final String TRANSMGR_PREALLOCATED_LOG_SIZE="transmgr.preallocated_log_size";
    public static final String TRANSMGR_CHECKPOINT_PERIOD="transmgr.checkpoint_period";
    public static final String TRANSMGR_WAIT_TIMEOUT="transmgr.wait_timeout";
    public static final String TRANSMGR_RETRY_TIMEOUT="transmgr.retry_timeout";
    public static final String TRANSMGR_DYNAMIC_RECLUSTERING_LIMIT="transmgr.dynamic_reclustering_limit";
    public static final String TRANSMGR_LOG_FILE_NAME="transmgr.log_file_name";
    public static final String TRANSMGR_HISTORY_FILE_NAME="transmgr.history_file_name";
    public static final String OBJMGR_LOCK_TIMEOUT="objmgr.lock_timeout";
    public static final String POOLMGR_PAGE_POOL_SIZE="poolmgr.page_pool_size";
    public static final String POOLMGR_DATA_FILE_NAME="poolmgr.data_file_name";
    public static final String SERVER_ADMIN_TELNET_PORT="server.admin_telnet_port";
    public static final String SERVER_CLUSTER_SIZE="server.cluster_size";

    public GoodsConfig (File cfgfile){
	this.cfgfile=cfgfile;
	readFile();
    }

    private void readFile() {
	if(!cfgfile.exists()) {
	    return;
	}

       	try { 
	    BufferedReader in = new BufferedReader(new FileReader(cfgfile));
	    String line;
	    String key;
	    String value;
	    RE stripcomment=new RE("(.*)#");
	    RE stripquotes=new RE("^['\"](.*?)['\"]$");
	    RE stripwhite=new RE("^\\s*(.*?)\\s*$");
	    RE getvalue=new RE("(.*?)=(.*)");
	    while((line=in.readLine())!=null) {
		if(stripcomment.match(line)) {
		    line=stripcomment.getParen(1);
		}
		if(getvalue.match(line)) {
		    key=getvalue.getParen(1);
		    value=getvalue.getParen(2);
		    if(stripquotes.match(key)) {
			key=stripquotes.getParen(1);
		    }
		    if(stripwhite.match(key)) {
			key=stripwhite.getParen(1);
		    }
		    if(stripquotes.match(value)) {
			value=stripquotes.getParen(1);
		    }
		    if(stripwhite.match(value)) {
			value=stripwhite.getParen(1);
		    }
		    setProperty(key, value);
		}
	    }
	} catch (Exception e) {
	    e.printStackTrace(System.err);
	}
    }

    public boolean isTelnetEnabled() {
	return (getProperty(SERVER_ADMIN_TELNET_PORT) != null);
    }
}// GoodsConfig
