package goodsutil;
/**
 * GoodsDbConfig.java
 *
 *
 * Created: Tue Dec  4 12:06:41 2001
 *
 * @author <a href="mailto:hotari@iki.fi">Lari Hotari</a>
 * @version
 */

import java.io.*;
import java.util.*;

public class GoodsDbConfig {
    public class StorageConfig {
	public String host;
	public int port;

	public String toString() {
	    return host + ":" + port;
	}
    }

    private StorageConfig[] storages;
    private File cfgfile;

    public class MyStreamTokenizer extends StreamTokenizer {
	public MyStreamTokenizer(InputStream in) {
	    super(in);
	    setNormalSyntax();
	}

	public void setNormalSyntax() {
	    resetSyntax();
	    wordChars('a', 'z');
	    wordChars('A', 'Z');
	    wordChars(128 + 32, 255);
	    whitespaceChars(0, ' ');
	    commentChar('/');
	    quoteChar('"');
	    quoteChar('\'');
	    parseNumbers();
	    wordChars('_','_');
	    whitespaceChars(':',':');
	}

	private void setHostnameSyntax() {
	    resetSyntax();
	    wordChars('a', 'z');
	    wordChars('A', 'Z');
	    wordChars(128 + 32, 255);
	    whitespaceChars(0, ' ');
	    commentChar('/');
	    quoteChar('"');
	    quoteChar('\'');
	    wordChars('_','_');
	    wordChars('.','.');
	    wordChars('0','9');
	    whitespaceChars(':',':');
	}
    }

    public GoodsDbConfig (File cfgfile){
	this.cfgfile=cfgfile;
	readFile();
    }

    public StorageConfig[] getStorageConfigs() {
	return storages;
    }

    public int getSize() {
	return storages.length;
    }
    
    public StorageConfig getStorageConfig(int snum) {
	return storages[snum];
    }

    public StorageConfig getStorageConfig() {
	return getStorageConfig(0);
    }

    private void readFile() {
    	try { 
	    FileInputStream in = new FileInputStream(cfgfile);
	    MyStreamTokenizer scanner = new MyStreamTokenizer(in);	    

	    if (scanner.nextToken() != StreamTokenizer.TT_NUMBER) { 
		in.close();
		return;
	    }
	    int nStorages = (int)scanner.nval;
	    storages = new StorageConfig[nStorages];
	    while (--nStorages >= 0) { 
		if (scanner.nextToken() != StreamTokenizer.TT_NUMBER) { 
		    in.close();
		    return;
		}
		int sid = (int)scanner.nval;
		if (sid < storages.length) { 
		    if (storages[sid] != null) { 
			in.close();
			return;
		    }
		    scanner.setHostnameSyntax();
		    int tkn = scanner.nextToken();
                    if (tkn != StreamTokenizer.TT_WORD 
                        && tkn != '"' && tkn != '\'')
                    { 
                        in.close();
                        return;
                    }
                    String hostname = scanner.sval;
		    scanner.setNormalSyntax();

		    if (scanner.nextToken() != StreamTokenizer.TT_NUMBER) { 
			in.close();
			return;
		    }
		    int port = (int)scanner.nval;
		    StorageConfig sconf=new StorageConfig();
		    sconf.host=hostname;
		    sconf.port=port;
		    storages[sid]=sconf;
		}
	    }
	    in.close();
	    return;
	} catch (IOException x) { 
	    return;
	}
    }

    public String toString() {
	StringBuffer sb=new StringBuffer();
	for(int i=0;i < storages.length;i++) {
	    sb.append(storages[i]);
	    if(i != storages.length-1) sb.append(",");
	}
	return sb.toString();
    }
}// GoodsDbConfig
