package goodsutil;
/**
 * GoodsLocalServer.java
 *
 *
 * Created: Tue Dec  4 12:06:41 2001
 *
 * @author <a href="mailto:hotari@iki.fi">Lari Hotari</a>
 * @version
 */

import java.io.*;
import java.util.*;

public class GoodsLocalServer implements GoodsServer {
    GoodsServerProcess[] servers;
    Properties goodsprops;
    File[] dbdirs;
    String dbname;
    String srvexecutable;

    public GoodsLocalServer (String dbname, Properties goodsprops, File[] dbdirs){
	this.dbdirs=dbdirs;
	this.dbname=dbname;
	this.goodsprops=goodsprops;
	initServers();
    }

    public GoodsLocalServer (String dbname, File goodsprops_file, File[] dbdirs){
	this.dbdirs=dbdirs;
	this.dbname=dbname;
	this.goodsprops=new Properties();
	try {
	    goodsprops.load(new FileInputStream(goodsprops_file));
	} catch (IOException e) {
	}
	initServers();
    }

    public GoodsLocalServer (String dbname, Properties goodsprops, File dbdir){
	this.dbdirs=new File[1];
	dbdirs[0]=dbdir;
	this.dbname=dbname;
	this.goodsprops=goodsprops;
	initServers();
    }

    public GoodsLocalServer (String dbname, File goodsprops_file, File dbdir){
	this.dbdirs=new File[1];
	dbdirs[0]=dbdir;
	this.dbname=dbname;
	this.goodsprops=new Properties();
	try {
	    goodsprops.load(new FileInputStream(goodsprops_file));
	} catch (IOException e) {
	}
	initServers();
    }

    private void initServers() {
	initExecutable();
	servers=new GoodsServerProcess[dbdirs.length];
	for(int i=0; i < dbdirs.length;i++) {
	    servers[i]=new GoodsServerProcess(srvexecutable, dbname, dbdirs[i], i);
	}
    }

    private void initExecutable() {
	String osname=getOSName();
	if(goodsprops != null) {
	    srvexecutable=goodsprops.getProperty("goodsdb.server." + osname);
	    if(srvexecutable==null && osname.indexOf("win") > -1) {
		srvexecutable=goodsprops.getProperty("goodsdb.server.windows");
	    }
	    if(srvexecutable==null) {
		if(osname.indexOf("win") > -1) {
		    srvexecutable="C:/goods/bin/goodsrv.exe";
		} else {
		    srvexecutable="/opt/goods/bin/goodsrv";
		}
	    }
	}
    }

    private String getOSName() {
	String osname=System.getProperty("os.name").toLowerCase();
	osname=osname.replace(' ','_');
	return osname;
    }

    public void start() throws Exception {
	for(int i=0;i < servers.length;i++) {
	    servers[i].start();
	}
	Thread.sleep(2000);
    }

    public void start(int snum) throws Exception {
	servers[snum].start();
	Thread.sleep(2000);
    }

    public void stop() throws Exception {
	for(int i=0;i < servers.length;i++) {
	    servers[i].stop();
	}
	Thread.sleep(2000);
    }

    public void stop(int snum) throws Exception {
	servers[snum].stop();
	Thread.sleep(2000);
    }

    public synchronized String executeCommand(int snum, String command) {
	return servers[snum].executeCommand(command);
    }

    public synchronized String executeCommand(String command) {
	if(servers.length > 1) {
	    StringBuffer sb=new StringBuffer();
	    for(int i=0;i < servers.length;i++) {
		sb.append("<serverresponse snum=\"").append(i).append("\">\n");
		sb.append(executeCommand(i, command));
		sb.append("<serverresponse />\n");
	    }
	    return sb.toString();
	} else {
	    return executeCommand(0, command);
	}
    }

    public GoodsServerProcess[] getProcesses() {
	return servers;
    }
}// GoodsLocalServer
