package goodsutil;
/**
 * GoodsServerProcess.java
 *
 *
 * Created: Tue Dec  4 12:06:41 2001
 *
 * @author <a href="mailto:hotari@iki.fi">Lari Hotari</a>
 * @version
 */

import java.io.*;

public class GoodsServerProcess {
    File dbdir;
    String dbname;
    int snum;
    GoodsDbConfig dbconfig;
    GoodsConfig srvconfig;
    Process srvprocess;
    String srvexecutable;
    OutputStream srv_input;
    boolean telnet_enabled;
    GoodsServerTelnetInterface telnet_if;

    public GoodsServerProcess (String srvexecutable, String dbname, File dbdir, int snum){
	this.srvexecutable=srvexecutable;
	this.dbdir=dbdir;
	this.dbname=dbname;
	this.snum=snum;
	dbconfig=new GoodsDbConfig(new File(dbdir, dbname + ".cfg"));
	srvconfig=new GoodsConfig(new File(dbdir, "goodsrv.cfg"));
    }

    public void start() throws IOException {
	String cmds[]=new String[3];
	cmds[0]=srvexecutable;
	cmds[1]=dbname;
	cmds[2]=String.valueOf(snum);
	srvprocess=Runtime.getRuntime().exec(cmds,null,dbdir);
	if(srvprocess != null) {
	    if(srvconfig.isTelnetEnabled()) {
		telnet_enabled=true;
		telnet_if=new GoodsServerTelnetInterface(srvconfig.getProperty(srvconfig.SERVER_ADMIN_TELNET_PORT));
	    } else {
		srv_input=srvprocess.getOutputStream();
	    }
	}
    }

    public synchronized String executeCommand(String command) {
	String response=null;
	if(telnet_enabled) {
	    try {
		telnet_if.connect();
		response=telnet_if.executeCommand(command);
		telnet_if.close();
	    } catch (IOException e) {
		e.printStackTrace(System.err);
	    }
	} else {
	    try {
		srv_input.write((command + "\n").getBytes());
		srv_input.flush();
	    } catch (IOException e) {
		e.printStackTrace(System.err);
	    }
	}
	return response;
    }

    public void stop() {
	if(srvprocess != null) {
	    executeCommand("exit");
	    System.out.println("Waiting for server to close...");
	    try {
		srvprocess.waitFor();
	    } catch (InterruptedException e) {
	    }
	    System.out.println("Server closed ok...");
	}
    }
}// GoodsServerProcess
