package goodsutil;
/**
 * GoodsServerProcessLogger.java
 *
 *
 * Created: Tue Dec  4 12:06:41 2001
 *
 * @author <a href="mailto:hotari@iki.fi">Lari Hotari</a>
 * @version
 */

import java.io.*;

public class GoodsServerProcessLogger extends Thread {
    BufferedOutputStream stdout_log;
    BufferedOutputStream stderr_log;
    BufferedInputStream stdout_in;
    BufferedInputStream stderr_in;
    BufferedOutputStream cmd_response;
    ByteArrayOutputStream cmd_response_bytes;
    boolean stoplogging=false;
    boolean running=false;
    boolean cmd_recording=false;
    byte lastbyte=0;
    boolean cmd_finished=false;
    

    public GoodsServerProcessLogger (int snum, BufferedOutputStream stdout_log,  BufferedOutputStream stderr_log, InputStream stdout_in, InputStream stderr_in){
	super("GoodsServerProcessLogger #" + snum);
	this.stdout_log=stdout_log;
	this.stderr_log=stderr_log;
	this.stdout_in=new BufferedInputStream(stdout_in);
	this.stderr_in=new BufferedInputStream(stderr_in);
    }

    public void stopLogging() {
	int count=0;
	stoplogging=true;
	while(running && count++ < 20) {
	    try {
		Thread.sleep(500);
	    } catch (InterruptedException e) {
	    }
	}
	synchronized(this) {
	    if(cmd_recording) {
		cmd_finished=true;
		notifyAll();
	    }
	}
    }

    public synchronized void startCommand() {
	cmd_response_bytes=new ByteArrayOutputStream();
	cmd_response=new BufferedOutputStream(cmd_response_bytes);
	cmd_recording=true;
	cmd_finished=false;
	lastbyte=0;
    }

    public synchronized String endCommand() {
	if(cmd_recording) {
	    cmd_recording=false;
	    try {
		cmd_response.close();
	    } catch (IOException e) {
	    }
	    String response=cmd_response_bytes.toString();
	    cmd_response=null;
	    cmd_response_bytes=null;
	    return response;
	} else {
	    return null;
	}
    }

    public synchronized boolean isCommandFinished() {
	if(!cmd_recording) return true;
	if(cmd_finished) return true;
	return false;
    }

    public synchronized String waitForCommand() {
	if(!cmd_recording) return null;
	if(cmd_finished) return null;
	try {
	    while(!cmd_finished) {
		wait();
	    }
	} catch (InterruptedException e) {
	    
	}
	return endCommand();
    }

    public void run() {
	byte[] cbuf_out=new byte[4096];
	int len_out=0;
	byte[] cbuf_err=new byte[4096];
	int len_err=0;

	try {
	    running=true;
	    boolean streamopen=true;
	    boolean streamread=false;
	    while(streamopen) {
		streamread=false;
		len_out=0;
		len_err=0;

		int read_out=stdout_in.available();
		if(read_out > cbuf_out.length) {
		    read_out=cbuf_out.length;
		}

		int read_err=stderr_in.available();
		if(read_err > cbuf_err.length) {
		    read_err=cbuf_err.length;
		}

		if(read_out > 0) {
		    len_out=stdout_in.read(cbuf_out, 0, read_out);
		}

		if(read_err > 0) {
		    len_err=stderr_in.read(cbuf_err, 0, read_err);
		}

		if(len_out > 0) {
		    System.out.print("X");
		    streamread=true;
		    stdout_log.write(cbuf_out,0,len_out);
		    boolean notify_after=false;
		    synchronized (this) {
			if(cmd_recording) {
			    if(!cmd_finished) {
				System.out.println("output-->" + new String(cbuf_out,0,len_out));
				for(int i=0;i < len_out;i++) {
				    if(i==0 && cbuf_out[i]=='>' && lastbyte=='\n') {
					cmd_finished=true;
					notify_after=true;
					break;
				    }
				    if(i < len_out-1 && cbuf_out[i]=='\n' && cbuf_out[i+1]=='>') {
					cmd_finished=true;
					notify_after=true;
					break;
				    }
				}
				lastbyte=cbuf_out[len_out-1];
			    }
			}
		    }
		    cmd_response.write(cbuf_out,0,len_out);
		    synchronized(this) {
			if(notify_after) {
			    notifyAll();
			}
		    }
		} 

		if(len_err > 0) {
		    streamread=true;
		    stderr_log.write(cbuf_err,0,len_err);
		}

		if(!streamread) {
		    if(stoplogging) {
			streamopen=false;
		    } else {
			try {
			    System.out.print("*SLEEP*");
			    Thread.sleep(500);
			    synchronized (stdout_in) {
				stdout_in.notifyAll();
			    }
			} catch (InterruptedException e) {
			}
		    }
		}
	    }
	    stdout_log.flush();
	    stderr_log.flush();
	    stdout_in.close();
	    stderr_in.close();
	} catch (IOException e) {
	    e.printStackTrace(System.err);
	} finally {
	    running=false;
	}
    }
}// GoodsServerProcessLogger
