package goodsutil;
/**
 * GoodsServerTelnetInterface.java
 *
 *
 * Created: Tue Dec  4 12:06:41 2001
 *
 * @author <a href="mailto:hotari@iki.fi">Lari Hotari</a>
 * @version
 */

import java.io.*;
import java.net.*;
import org.apache.regexp.*;

public class GoodsServerTelnetInterface {
    String host;
    int port;
    Socket socket;
    TelnetInputThread input_thread;
    DataOutputStream output;

    private class TelnetInputThread extends Thread {
	DataInputStream telnet_input;
	StringBuffer sb;
	boolean recording=false;

	public TelnetInputThread(DataInputStream telnet_input) {
	    this.telnet_input=telnet_input;
	}

	public synchronized void close() throws IOException {
	    telnet_input.close();
	}

	public synchronized void startCommand() {
	    this.sb=new StringBuffer();
	    recording=true;
	}

	public synchronized String waitForCommand() {
	    if(recording) {
		try {
		    this.wait();
		} catch (InterruptedException e) {
		}
	    } 
	    String response=null;
	    if(sb != null) {
		response=sb.toString();
		sb=null;
	    }
	    return response;
	}
	
	public void run() {
	    startCommand();
	    try {
		byte b=0;
		byte last=0;
		while((b=telnet_input.readByte()) != -1) {
		    synchronized(this) {
			if(recording) {
			    if((last=='\n' || last==0 || last=='\r') && b=='>') {
				recording=false;
				this.notify();
			    }
			    sb.append((char)b);
			}
		    }
		}
	    } catch (IOException e) {

	    } finally {
		synchronized(this) {
		    if(recording) {
			recording=false;
			this.notify();
		    }
		}
	    }
	}
    }
    

    public GoodsServerTelnetInterface (String address) {
	try {
	    RE re=new RE("(\\S+?):(\\d+)");
	    if(re.match(address)) {
		host=re.getParen(1);
		port=Integer.parseInt(re.getParen(2));
	    }
	} catch (RESyntaxException e) {
	    e.printStackTrace(System.err);
	}
    }

    public void connect() throws IOException, UnknownHostException {
	socket=new Socket(host,port);
	input_thread=new TelnetInputThread(new DataInputStream(socket.getInputStream()));
	input_thread.start();
	output=new DataOutputStream(socket.getOutputStream());
	try {
	    Thread.sleep(500);
	} catch (InterruptedException e) {
	}
	String response=input_thread.waitForCommand();
    }

    public String executeCommand(String command) throws IOException {
	input_thread.startCommand();
	output.writeBytes(command + "\n");
	output.flush();

	String response=input_thread.waitForCommand();

	if(!command.equals("exit")) {
	    output.writeBytes("logout\n");
	    output.flush();
	}
	return response.toString();
    }

    public void close() throws IOException {
	output.close();
	socket.close();
	input_thread.close();
	output=null;
	input_thread=null;
	socket=null;
    }
    
}// GoodsServerTelnetInterface
