
/**
 * $Id: ListAdapter.java,v 1.1 2001/10/04 14:52:04 lari Exp $
 * ListAdapter.java
 *
 *
 * Created: Tue Dec  4 12:06:41 2001
 *
 * @author <a href="mailto: hotari@iki.fi">Lari Hotari</a>
 * @version
 */

package oodblib.adapter;

import java.util.*;
import java.io.*;

public class ListAdapter extends AbstractAdapter implements java.util.List, MappedObject, Serializable {
    static final long serialVersionUID = -6008698461346880815L; //Generated by jserial
    transient private List datalist;

    public void writeObject(ObjectOutputStream out) throws IOException {
	super.writeObject(out);
	if(datalist instanceof java.io.Serializable) {
	    out.writeObject(datalist);
	} else {
	    out.writeObject(null);
	}
    }

    public void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
	super.readObject(in);
	datalist=(List)in.readObject();
    }

    public ListAdapter (ObjectMapper mapper, List datalist) {
	super(mapper);
	this.datalist=datalist;
	this.convertdataobjects=true;
	mapper.registerObject(this);
    }

    public ListAdapter (ObjectMapper mapper, List datalist, boolean convertdataobjects) {
	super(mapper);
	this.datalist=datalist;
	this.convertdataobjects=convertdataobjects;
	mapper.registerObject(this);
    }

    public Object getDataObject() {
	return datalist;
    }

    public void add(int index, Object element) {
	datalist.add(index, toPersistentObject(element));
    }

    public boolean add(Object o) {
	return datalist.add(toPersistentObject(o));
    }

    public boolean addAll(Collection c) {
	for(Iterator it=c.iterator();it.hasNext();) {
	    datalist.add(toPersistentObject(it.next()));
	}
	return true;
    }
	
    public boolean addAll(int index, Collection c) {
	int pos=index;
	for(Iterator it=c.iterator();it.hasNext();pos++) {
	    datalist.add(pos, toPersistentObject(it.next()));
	}
	return true;
    }

    public void clear() {
	datalist.clear();
    }
   
    public boolean contains(Object elem) {
	return datalist.contains(toPersistentObject(elem));
    }

    public Object get(int index) {
	try {
	    return toNormalObject(datalist.get(index));
	} catch (Exception e) {
	    return null;
	}
    }

    public int indexOf(Object elem) {
	return datalist.indexOf(toPersistentObject(elem));
    }

    public boolean isEmpty() {
	return datalist.isEmpty();
    }

    public int lastIndexOf(Object elem) {
	return datalist.lastIndexOf(toPersistentObject(elem));
    }

    public Object remove(int index) {
	return toNormalObject(datalist.remove(index));
    }

    public Object set(int index, Object element) {
	return toNormalObject(datalist.set(index, toPersistentObject(element)));
    }

    public int size() {
	return datalist.size();
    }

    public Object[] toArray() {
	Object[] dataobjs=datalist.toArray();
	Object[] objs=(Object [])dataobjs.clone();
	for(int i=0;i < dataobjs.length;i++) {
	    objs[i]=toNormalObject(dataobjs[i]);
	}
	return objs;
    }

    public Object[] toArray(Object[] a) {
	Object[] dataobjs=datalist.toArray(a);
	Object[] objs=(Object [])dataobjs.clone();
	for(int i=0;i < dataobjs.length;i++) {
	    objs[i]=toNormalObject(dataobjs[i]);
	}
	return objs;
    }

    public Iterator iterator() {
	return new IteratorAdapter(datalist.iterator(),this);
    }

    public List subList(int fromIndex, int toIndex) {
	return datalist.subList(fromIndex, toIndex);
    }

    public boolean containsAll(Collection c) {
	return datalist.containsAll(c);
    }
    public boolean remove(Object o) {
	return datalist.remove(toPersistentObject(o));
    }

    public boolean removeAll(Collection c) {
	return datalist.removeAll(c);
    }

    public boolean retainAll(Collection c) {
	return datalist.retainAll(c);
    }

    public ListIterator listIterator() {
	return new ListIteratorAdapter(datalist.listIterator(),this);
    }
    
    public ListIterator listIterator(int i) {
	return new ListIteratorAdapter(datalist.listIterator(i),this);
    }

    public int hashCode() {
	return datalist.hashCode();
    }
}// ListAdapter

