
/**
 * $Id: MapAdapter.java,v 1.1 2001/10/04 14:52:04 lari Exp $
 * MapAdapter.java
 *
 *
 * Created: Tue Dec  4 12:06:41 2001
 *
 * @author <a href="mailto: hotari@iki.fi">Lari Hotari</a>
 * @version
 */

package oodblib.adapter;

import oodblib.*;
import java.util.*;
import java.io.*;

public class MapAdapter extends AbstractAdapter implements java.util.Map, MappedObject, Serializable {
    static final long serialVersionUID = 1697739901008298002L; //Generated by jserial
    transient private Map datamap;

    public void writeObject(ObjectOutputStream out) throws IOException {
	super.writeObject(out);
	if(datamap instanceof java.io.Serializable) {
	    out.writeObject(datamap);
	} else {
	    out.writeObject(null);
	}
    }

    public void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
	super.readObject(in);
	datamap=(Map)in.readObject();
    }

    public MapAdapter (ObjectMapper mapper, Map datamap) {
	super(mapper);
	this.datamap=datamap;
	this.convertdataobjects=true;
	mapper.registerObject(this);
    }

    public MapAdapter (ObjectMapper mapper, Map datamap, boolean convertdataobjects) {
	super(mapper);
	this.datamap=datamap;
	this.convertdataobjects=convertdataobjects;
	mapper.registerObject(this);
    }

    public Object getDataObject() {
	return datamap;
    }

    public void clear() {
	datamap.clear();
    }  
    
    public boolean containsKey(Object key) {
	return datamap.containsKey(toPersistentObject(key));
    }
    
    public boolean containsValue(Object value) {
	return datamap.containsValue(toPersistentObject(value));
    }
    
    public Set entrySet() {
	return new CollectionAdapter(datamap.entrySet(), this);
    }

    public Object get(Object key) {
	return toNormalObject(datamap.get(toPersistentObject(key)));
    }

    public boolean isEmpty() {
	return datamap.isEmpty();
    }

    public Set keySet() {
	return new CollectionAdapter(datamap.keySet(), this);
    }
    
    public Object put(Object key, Object value) { 
	Object converted_key=toPersistentObject(key);
	Object converted_val=toPersistentObject(value);
	return toNormalObject(datamap.put(converted_key, converted_val));
    }

    public void putAll(Map t) {
	if(t != null) {
	    for(Iterator it=t.entrySet().iterator();it.hasNext();) {
		Map.Entry entry=(Map.Entry)it.next();
		put(toPersistentObject(entry.getKey()),toPersistentObject(entry.getValue()));
	    }
	}
    }

    public Object remove(Object key) {
	return toNormalObject(datamap.remove(toPersistentObject(key)));
    }

    public int size() {
	return datamap.size();
    }

    public Collection values() {
	return new CollectionAdapter(datamap.values(), this);
    }

    public int hashCode() {
	return datamap.hashCode();
    }
}// MapAdapter

