
/**
 * $Id: StackAdapter.java,v 1.1 2001/10/04 14:52:04 lari Exp $
 * StackAdapter.java
 *
 *
 * Created: Tue Dec  4 12:06:41 2001
 *
 * @author <a href="mailto: hotari@iki.fi">Lari Hotari</a>
 * @version
 */

package oodblib.adapter;

import goodsutil.Stack;
import java.util.*;
import java.io.*;

public class StackAdapter extends AbstractAdapter implements goodsutil.Stack, MappedObject, Serializable {
    static final long serialVersionUID = 9020288957593351690L; //Generated by jserial
    transient private Stack datastack;
    
    public void writeObject(ObjectOutputStream out) throws IOException {
	super.writeObject(out);
	if(datastack instanceof java.io.Serializable) {
	    out.writeObject(datastack);
	} else {
	    out.writeObject(null);
	}
    }

    public void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
	super.readObject(in);
	datastack=(Stack)in.readObject();
    }

    public StackAdapter (ObjectMapper mapper, Stack datastack) {
	super(mapper);
	this.datastack=datastack;
	this.convertdataobjects=true;
	mapper.registerObject(this);
    }

    public StackAdapter (ObjectMapper mapper, Stack datastack, boolean convertdataobjects) {
	super(mapper);
	this.datastack=datastack;
	this.convertdataobjects=convertdataobjects;
	mapper.registerObject(this);
    }

    public Object getDataObject() {
	return datastack;
    }

    public void add(int index, Object element) {
	datastack.add(index, toPersistentObject(element));
    }

    public boolean add(Object o) {
	return datastack.add(toPersistentObject(o));
    }

    public boolean addAll(Collection c) {
	for(Iterator it=c.iterator();it.hasNext();) {
	    datastack.add(toPersistentObject(it.next()));
	}
	return true;
    }
	
    public boolean addAll(int index, Collection c) {
	int pos=index;
	for(Iterator it=c.iterator();it.hasNext();pos++) {
	    datastack.add(pos, toPersistentObject(it.next()));
	}
	return true;
    }

    public void clear() {
	datastack.clear();
    }
   
    public boolean contains(Object elem) {
	return datastack.contains(toPersistentObject(elem));
    }

    public Object get(int index) {
	return toNormalObject(datastack.get(index));
    }

    public int indexOf(Object elem) {
	return datastack.indexOf(toPersistentObject(elem));
    }

    public boolean isEmpty() {
	return datastack.isEmpty();
    }

    public int lastIndexOf(Object elem) {
	return datastack.lastIndexOf(toPersistentObject(elem));
    }

    public Object remove(int index) {
	return toNormalObject(datastack.remove(index));
    }

    public Object set(int index, Object element) {
	return toNormalObject(datastack.set(index, toPersistentObject(element)));
    }

    public int size() {
	return datastack.size();
    }

    public Object[] toArray() {
	Object[] dataobjs=datastack.toArray();
	Object[] objs=(Object [])dataobjs.clone();
	for(int i=0;i < dataobjs.length;i++) {
	    objs[i]=toNormalObject(dataobjs[i]);
	}
	return objs;
    }

    public Object[] toArray(Object[] a) {
	Object[] dataobjs=datastack.toArray(a);
	Object[] objs=(Object [])dataobjs.clone();
	for(int i=0;i < dataobjs.length;i++) {
	    objs[i]=toNormalObject(dataobjs[i]);
	}
	return objs;
    }

    public Iterator iterator() {
	return new IteratorAdapter(datastack.iterator(),this);
    }

    public List subList(int fromIndex, int toIndex) {
	return datastack.subList(fromIndex, toIndex);
    }

    public boolean containsAll(Collection c) {
	return datastack.containsAll(c);
    }
    public boolean remove(Object o) {
	return datastack.remove(toPersistentObject(o));
    }

    public boolean removeAll(Collection c) {
	return datastack.removeAll(c);
    }

    public boolean retainAll(Collection c) {
	return datastack.retainAll(c);
    }

    public ListIterator listIterator() {
	return new ListIteratorAdapter(datastack.listIterator(),this);
    }
    
    public ListIterator listIterator(int i) {
	return new ListIteratorAdapter(datastack.listIterator(i),this);
    }

    public int hashCode() {
	return datastack.hashCode();
    }

    public boolean empty() {
	return datastack.empty();
    }

    public Object peek() throws EmptyStackException {
	if(empty()) throw new EmptyStackException();
	return toNormalObject(datastack.peek());
    }

    public Object pop() throws EmptyStackException {
	if(empty()) throw new EmptyStackException();
	return toNormalObject(datastack.pop());
    }

    public Object push(Object item) {
	return toNormalObject(datastack.push(toPersistentObject(item)));
    }

    public int search(Object o) {
	return datastack.search(toPersistentObject(o));
    }
}// StackAdapter

